using System;

namespace NsBusinessLogic
{
	/// <summary>
	/// Summary description for Delivery.
	/// </summary>
	public class Delivery
	{
		protected NsPersistent.PDelivery mPersistent = null;

		public Delivery(NsPersistent.PDelivery persistent)
		{
			mPersistent = persistent;
		}


		//******************** Business logic methods ******************

		public void GenerateDeliveryReport()
		{
			DeliveryReport report = new DeliveryReport(
				mPersistent.GetOrder().OrderId, 
				mPersistent.GetOrder().GetCustomer().Name,
				mPersistent.GetOrder().GetCustomer().Address,
				mPersistent.DeliveryDate);

			DeliveryItem [] aItemDeliveries = GetItemDeliveries();

			foreach (DeliveryItem deliveryItem in aItemDeliveries)
			{
				NsPersistent.PProduct product = deliveryItem.GetOrderItem().GetProduct().Persistent;
				report.AddProductElement(product.ProductId, product.Description, deliveryItem.Qty);
			}

			report.Save("Order " + mPersistent.GetOrder().OrderId + " Delivery " + mPersistent.DeliveryId + ".xml");
		}


		//**************** Persistent properties & Navigation methods ****************

		public NsPersistent.PDelivery Persistent
		{
			get { return mPersistent; }
		}

		public DeliveryItem AddItem(OrderItem orderItem, int iQty)
		{
			return new DeliveryItem(mPersistent.AddItem(orderItem.Persistent, iQty));
		}

		public DeliveryItem [] GetItemDeliveries()
		{
			NsPersistent.PDeliveryItem [] aPersistents = mPersistent.GetItemDeliveries();
			DeliveryItem [] aItemDeliveries = new DeliveryItem[aPersistents.Length];

			for (int i = 0; i < aPersistents.Length; i++)
			{
				aItemDeliveries[i] = new DeliveryItem(aPersistents[i]);
			}

			return aItemDeliveries;
		}
	}
}
