using System;
using System.Xml;

namespace NsBusinessLogic
{
	/* This class generates xml document as shown below:
	 * 
	<OrderDeliveryReport Customer="Varian" Address="679 Springvale Rd, Mulgrave, Vic 3150, Australia" Date="25/12/2003">
		<Product Id="3" Description="MP3 player flash memory 64M" Qty="10"/>
		<Product Id="5" Description="MP3 player flash memory FM radio 256M" Qty="10"/>
	</OrderDeliveryReport>	 
	*/

	public class DeliveryReport
	{
		private XmlDocument mDoc = new XmlDocument();

		public DeliveryReport(int iOrderId, string strCustomer, string strAddress, DateTime date)
		{
			mDoc.LoadXml("<DeliveryReport/>");

			XmlAttribute attrib = mDoc.CreateAttribute("OrderId");
			attrib.Value = Convert.ToString(iOrderId);
			mDoc.DocumentElement.SetAttributeNode(attrib);

			attrib = mDoc.CreateAttribute("Customer");
			attrib.Value = strCustomer;
			mDoc.DocumentElement.SetAttributeNode(attrib);

			attrib = mDoc.CreateAttribute("Address");
			attrib.Value = strAddress;
			mDoc.DocumentElement.SetAttributeNode(attrib);

			attrib = mDoc.CreateAttribute("Date");
			attrib.Value = date.ToShortDateString();
			mDoc.DocumentElement.SetAttributeNode(attrib);
		}

		public void AddProductElement(int iProductId, string strDescrip, int iOrderQty)
		{
			XmlElement elem = mDoc.CreateElement("Product");
			mDoc.DocumentElement.AppendChild(elem);

			XmlAttribute attrib = mDoc.CreateAttribute("Id");
			attrib.Value = Convert.ToString(iProductId);
			elem.Attributes.Append(attrib);

			attrib = mDoc.CreateAttribute("Description");
			attrib.Value = strDescrip;
			elem.Attributes.Append(attrib);

			attrib = mDoc.CreateAttribute("Qty");
			attrib.Value = Convert.ToString(iOrderQty);
			elem.Attributes.Append(attrib);
		}

		public void Save(string strFilePath)
		{
			mDoc.Save(strFilePath);
		}

		public string GetXmlString()
		{
			return mDoc.OuterXml;
		}
	}
}
