using System;

namespace NsBusinessLogic
{
	public class OrderItem
	{
		protected NsPersistent.POrderItem mPersistent = null;

		public OrderItem(NsPersistent.POrderItem persistent)
		{
			mPersistent = persistent;
		}


		//******************** Business logic methods ******************

		/// <summary>
		/// Compare the stock of the product with the minumum delivery qty of the order item to
		/// decide whether to delivery one batch. If yes, deduct the delivery qty from the product's
		/// stocked qty and add delivery item under the delivery. 
		/// </summary>
		public void CheckForDelivery()
		{
			int iToBeDeliveredQty = Math.Min(mPersistent.PendingQty, mPersistent.MinimumDeliveryQty);

			if (GetProduct().StockedOty >= iToBeDeliveredQty)
			{
				GetProduct().AddStock(-iToBeDeliveredQty);
				GetOrder().CurrentDelivery.AddItem(this, iToBeDeliveredQty);
				mPersistent.PendingQty -= iToBeDeliveredQty;
			}
		}


		//**************** Persistent properties & Navigation methods ****************

		public NsPersistent.POrderItem Persistent
		{
			get { return mPersistent; }
		}

		public int PendingQty
		{
			get { return mPersistent.PendingQty; }
		}

		public Order GetOrder()
		{
			return new Order(mPersistent.GetOrder());
		}

		public Product GetProduct()
		{
			return new Product(mPersistent.GetProduct());
		}
	}
}
