using System;
using System.Data;
using System.Data.OleDb;
using System.Data.Common;


namespace NsDataAccess
{
	/// <summary>
	/// CommunicatorSqlSever uses ADO.NET OLEDB data provide. It uses batch queries to get back 
	/// SQL server's auto-increment columns. 
	/// </summary>
	public class CommunicatorSqlServer : Communicator
	{
		private OleDbConnection mcn = null;

		private OleDbDataAdapter mdaCustomer = null;
		private OleDbDataAdapter mdaOrder = null;
		private OleDbDataAdapter mdaProduct = null;
		private OleDbDataAdapter mdaOrderItem = null;
		private OleDbDataAdapter mdaDelivery = null;
		private OleDbDataAdapter mdaDeliveryItem = null;

		private void BuildCustomerDataAdapter()
		{
			OleDbCommand cmd = null;

			mdaCustomer = new OleDbDataAdapter();
			mdaCustomer.TableMappings.AddRange(new DataTableMapping[] {
				new DataTableMapping("Table", "Customer", new DataColumnMapping[] {
				new DataColumnMapping("CustomerId", "CustomerId"),
				new DataColumnMapping("Name", "Name"),
				new DataColumnMapping("Address", "Address")})});

			// ------------------ SELECT -------------------
			cmd = new OleDbCommand();
			cmd.Connection = mcn;
			cmd.CommandText = "SELECT * FROM Customer";
			mdaCustomer.SelectCommand = cmd;

			// ------------------ INSERT -------------------
			cmd = new OleDbCommand();
			cmd.Connection = mcn;
			cmd.CommandText = "INSERT INTO Customer VALUES (?, ?); SELECT SCOPE_IDENTITY() AS CustomerId"; // "SCOPE_IDENTITY()" is the auto-generated identity.
			cmd.Parameters.Add(new OleDbParameter("Name", OleDbType.VarChar, 50, "Name"));
			cmd.Parameters.Add(new OleDbParameter("Address", OleDbType.VarChar, 50, "Address"));
			mdaCustomer.InsertCommand = cmd;

			// ------------------ UPDATE -------------------
			cmd = new OleDbCommand();
			cmd.Connection = mcn;
			cmd.CommandText = "UPDATE Customer SET Name = ?, Address = ? WHERE CustomerId = ?"; 
			cmd.Parameters.Add(new OleDbParameter("Name", OleDbType.VarChar, 50, "Name"));
			cmd.Parameters.Add(new OleDbParameter("Address", OleDbType.VarChar, 50, "Address"));
			cmd.Parameters.Add(new OleDbParameter("Original_CustomerId", OleDbType.Integer, 4, System.Data.ParameterDirection.Input, false, ((System.Byte)(0)), ((System.Byte)(0)), "CustomerId", System.Data.DataRowVersion.Original, null));
			mdaCustomer.UpdateCommand = cmd;

			// ------------------ DELETE -------------------
			cmd = new OleDbCommand();
			cmd.Connection = mcn;
			cmd.CommandText = "DELETE FROM Customer WHERE CustomerId = ?";
			cmd.Parameters.Add(new OleDbParameter("Original_CustomerId", OleDbType.Integer, 4, System.Data.ParameterDirection.Input, false, ((System.Byte)(0)), ((System.Byte)(0)), "CustomerId", System.Data.DataRowVersion.Original, null));
			mdaCustomer.DeleteCommand = cmd;
		}

		private void BuildOrderDataAdapter()
		{
			OleDbCommand cmd = null;

			mdaOrder = new OleDbDataAdapter();
			mdaOrder.TableMappings.AddRange(new DataTableMapping[] {
				new DataTableMapping("Table", "Order", new DataColumnMapping[] {
				new DataColumnMapping("OrderId", "OrderId"),
				new DataColumnMapping("CustomerId", "CustomerId"),
				new DataColumnMapping("OrderDate", "OrderDate"),
				new DataColumnMapping("TotalCharge", "TotalCharge"),
				new DataColumnMapping("Closed", "Closed")})});

			// ------------------ SELECT -------------------
			cmd = new OleDbCommand();
			cmd.Connection = mcn;
			cmd.CommandText = "SELECT * FROM [Order]";
			mdaOrder.SelectCommand = cmd;

			// ------------------ INSERT -------------------
			cmd = new OleDbCommand();
			cmd.Connection = mcn;
			cmd.CommandText = "INSERT INTO [Order] VALUES (?, ?, ?, ?); SELECT SCOPE_IDENTITY() AS OrderId"; // "SCOPE_IDENTITY()" is the auto-generated identity.
			cmd.Parameters.Add(new OleDbParameter("CustomerId", OleDbType.Integer, 4, "CustomerId"));
			cmd.Parameters.Add(new OleDbParameter("OrderDate", OleDbType.Date, 8, "OrderDate"));
			cmd.Parameters.Add(new OleDbParameter("TotalCharge", OleDbType.Currency, 8, "TotalCharge"));
			cmd.Parameters.Add(new OleDbParameter("Closed", OleDbType.Boolean, 1, "Closed"));
			mdaOrder.InsertCommand = cmd;

			// ------------------ UPDATE -------------------
			cmd = new OleDbCommand();
			cmd.Connection = mcn;
			cmd.CommandText = "UPDATE [Order] SET CustomerId = ?, OrderDate = ?, TotalCharge = ?, Closed = ? WHERE OrderId = ?"; 
			cmd.Parameters.Add(new OleDbParameter("CustomerId", OleDbType.Integer, 4, "CustomerId"));
			cmd.Parameters.Add(new OleDbParameter("OrderDate", OleDbType.Date, 8, "OrderDate"));
			cmd.Parameters.Add(new OleDbParameter("TotalCharge", OleDbType.Currency, 8, "TotalCharge"));
			cmd.Parameters.Add(new OleDbParameter("Closed", OleDbType.Boolean, 1, "Closed"));
			cmd.Parameters.Add(new OleDbParameter("Original_OrderId", OleDbType.Integer, 4, System.Data.ParameterDirection.Input, false, ((System.Byte)(0)), ((System.Byte)(0)), "OrderId", System.Data.DataRowVersion.Original, null));
			mdaOrder.UpdateCommand = cmd;

			// ------------------ DELETE -------------------
			cmd = new OleDbCommand();
			cmd.Connection = mcn;
			cmd.CommandText = "DELETE FROM [Order] WHERE OrderId = ?";
			cmd.Parameters.Add(new OleDbParameter("Original_OrderId", OleDbType.Integer, 4, System.Data.ParameterDirection.Input, false, ((System.Byte)(0)), ((System.Byte)(0)), "OrderId", System.Data.DataRowVersion.Original, null));
			mdaOrder.DeleteCommand = cmd;
		}

		private void BuildProductDataAdapter()
		{
			OleDbCommand cmd = null;

			mdaProduct = new OleDbDataAdapter();
			mdaProduct.TableMappings.AddRange(new DataTableMapping[] {
				new DataTableMapping("Table", "Product", new DataColumnMapping[] {
				new DataColumnMapping("ProductId", "ProductId"),
				new DataColumnMapping("Description", "Description"),
				new DataColumnMapping("Price", "Price"),
				new DataColumnMapping("StockedQty", "StockedQty")})});

			// ------------------ SELECT -------------------
			cmd = new OleDbCommand();
			cmd.Connection = mcn;
			cmd.CommandText = "SELECT * FROM Product";
			mdaProduct.SelectCommand = cmd;

			// ------------------ INSERT -------------------
			cmd = new OleDbCommand();
			cmd.Connection = mcn;
			cmd.CommandText = "INSERT INTO Product VALUES (?, ?, ?); SELECT SCOPE_IDENTITY() AS ProductId";
			cmd.Parameters.Add(new OleDbParameter("Description", OleDbType.VarChar, 50, "Description"));
			cmd.Parameters.Add(new OleDbParameter("Price", OleDbType.Currency, 4, "Price"));
			cmd.Parameters.Add(new OleDbParameter("StockedQty", OleDbType.Integer, 4, "StockedQty"));
			mdaProduct.InsertCommand = cmd;

			// ------------------ UPDATE -------------------
			cmd = new OleDbCommand();
			cmd.Connection = mcn;
			cmd.CommandText = "UPDATE Product SET Description = ?, Price = ?, StockedQty = ? WHERE ProductId = ?"; 
			cmd.Parameters.Add(new OleDbParameter("Description", OleDbType.VarChar, 50, "Description"));
			cmd.Parameters.Add(new OleDbParameter("Price", OleDbType.Currency, 4, "Price"));
			cmd.Parameters.Add(new OleDbParameter("StockedQty", OleDbType.Integer, 4, "StockedQty"));
			cmd.Parameters.Add(new OleDbParameter("Original_ProductId", OleDbType.Integer, 4, System.Data.ParameterDirection.Input, false, ((System.Byte)(0)), ((System.Byte)(0)), "ProductId", System.Data.DataRowVersion.Original, null));
			mdaProduct.UpdateCommand = cmd;

			// ------------------ DELETE -------------------
			cmd = new OleDbCommand();
			cmd.Connection = mcn;
			cmd.CommandText = "DELETE FROM Product WHERE ProductId = ?";
			cmd.Parameters.Add(new OleDbParameter("Original_ProductId", OleDbType.Integer, 4, System.Data.ParameterDirection.Input, false, ((System.Byte)(0)), ((System.Byte)(0)), "ProductId", System.Data.DataRowVersion.Original, null));
			mdaProduct.DeleteCommand = cmd; 
		}

		private void BuildOrderItemDataAdapter()
		{
			OleDbCommand cmd = null;

			mdaOrderItem = new OleDbDataAdapter();
			mdaOrderItem.TableMappings.AddRange(new System.Data.Common.DataTableMapping[] {
				new System.Data.Common.DataTableMapping("Table", "OrderItem", new System.Data.Common.DataColumnMapping[] {
				new System.Data.Common.DataColumnMapping("OrderItemId", "OrderItemId"),
				new System.Data.Common.DataColumnMapping("OrderId", "OrderId"),
				new System.Data.Common.DataColumnMapping("ProductId", "ProductId"),
				new System.Data.Common.DataColumnMapping("Qty", "Qty"),
				new System.Data.Common.DataColumnMapping("MinDeliveryQty", "MinDeliveryQty"),
				new System.Data.Common.DataColumnMapping("PendingQty", "PendingQty")})});

			// ------------------ SELECT -------------------
			cmd = new OleDbCommand();
			cmd.Connection = mcn;
			cmd.CommandText = "SELECT * FROM OrderItem";
			mdaOrderItem.SelectCommand = cmd;

			// ------------------ INSERT -------------------
			cmd = new OleDbCommand();
			cmd.Connection = mcn;
			cmd.CommandText = "INSERT INTO OrderItem VALUES (?, ?, ?, ?, ?, ?); SELECT SCOPE_IDENTITY() AS OrderItemId";
			cmd.Parameters.Add(new OleDbParameter("OrderId", OleDbType.Integer, 4, "OrderId"));
			cmd.Parameters.Add(new OleDbParameter("ProductId", OleDbType.Integer, 4, "ProductId"));
			cmd.Parameters.Add(new OleDbParameter("Qty", OleDbType.Integer, 4, "Qty"));
			cmd.Parameters.Add(new OleDbParameter("MinDeliveryQty", OleDbType.Integer, 4, "MinDeliveryQty"));
			cmd.Parameters.Add(new OleDbParameter("PendingQty", OleDbType.Integer, 4, "PendingQty"));
			mdaOrderItem.InsertCommand = cmd;

			// ------------------ UPDATE -------------------
			cmd = new OleDbCommand();
			cmd.Connection = mcn;
			cmd.CommandText = "UPDATE OrderItem SET OrderId = ?, ProductId = ?, Qty = ?, MinDeliveryQty = ?, PendingQty = ? WHERE OrderItemId = ?";
			cmd.Parameters.Add(new OleDbParameter("OrderId", OleDbType.Integer, 4, "OrderId"));
			cmd.Parameters.Add(new OleDbParameter("ProductId", OleDbType.Integer, 4, "ProductId"));
			cmd.Parameters.Add(new OleDbParameter("Qty", OleDbType.Integer, 4, "Qty"));
			cmd.Parameters.Add(new OleDbParameter("MinDeliveryQty", OleDbType.Integer, 4, "MinDeliveryQty"));
			cmd.Parameters.Add(new OleDbParameter("PendingQty", OleDbType.Integer, 4, "PendingQty"));
			cmd.Parameters.Add(new OleDbParameter("Original_OrderItemId", OleDbType.Integer, 4, System.Data.ParameterDirection.Input, false, ((System.Byte)(0)), ((System.Byte)(0)), "OrderItemId", System.Data.DataRowVersion.Original, null));
			mdaOrderItem.UpdateCommand = cmd;

			// ------------------ DELETE -------------------
			cmd = new OleDbCommand();
			cmd.Connection = mcn;
			cmd.CommandText = "DELETE FROM OrderItem WHERE OrderItemId = ?";
			cmd.Parameters.Add(new OleDbParameter("Original_OrderItemId", OleDbType.Integer, 4, System.Data.ParameterDirection.Input, false, ((System.Byte)(0)), ((System.Byte)(0)), "OrderItemId", System.Data.DataRowVersion.Original, null));
			mdaOrderItem.DeleteCommand = cmd;
		}

		private void BuildDeliveryDataAdapter()
		{
			OleDbCommand cmd = null;

			mdaDelivery = new OleDbDataAdapter();
			mdaDelivery.TableMappings.AddRange(new DataTableMapping[] {
				new DataTableMapping("Table", "Delivery", new DataColumnMapping[] {
				new DataColumnMapping("DeliveryId", "DeliveryId"),
				new DataColumnMapping("OrderId", "OrderId"),
				new DataColumnMapping("DeliveryDate", "DeliveryDate"),
				new DataColumnMapping("Address", "Address")})});

			// ------------------ SELECT -------------------
			cmd = new OleDbCommand();
			cmd.Connection = mcn;
			cmd.CommandText = "SELECT * FROM Delivery";
			mdaDelivery.SelectCommand = cmd;

			// ------------------ INSERT -------------------
			cmd = new OleDbCommand();
			cmd.Connection = mcn;
			cmd.CommandText = "INSERT INTO Delivery VALUES (?, ?, ?); SELECT SCOPE_IDENTITY() AS DeliveryId";
			cmd.Parameters.Add(new OleDbParameter("OrderId", OleDbType.Integer, 4, "OrderId"));
			cmd.Parameters.Add(new OleDbParameter("DeliveryDate", OleDbType.Date, 8, "DeliveryDate"));
			cmd.Parameters.Add(new OleDbParameter("Address", OleDbType.VarChar, 50, "Address"));
			mdaDelivery.InsertCommand = cmd;

			// ------------------ UPDATE -------------------
			cmd = new OleDbCommand();
			cmd.Connection = mcn;
			cmd.CommandText = "UPDATE Delivery SET OrderId = ?, DeliveryDate = ?, Address = ? WHERE DeliveryId = ?";
			cmd.Parameters.Add(new OleDbParameter("OrderId", OleDbType.Integer, 4, "OrderId"));
			cmd.Parameters.Add(new OleDbParameter("DeliveryDate", OleDbType.Date, 8, "DeliveryDate"));
			cmd.Parameters.Add(new OleDbParameter("Address", OleDbType.VarChar, 50, "Address"));
			cmd.Parameters.Add(new OleDbParameter("Original_DeliveryId", OleDbType.Integer, 4, System.Data.ParameterDirection.Input, false, ((System.Byte)(0)), ((System.Byte)(0)), "DeliveryId", System.Data.DataRowVersion.Original, null));
			mdaDelivery.UpdateCommand = cmd;

			// ------------------ DELETE -------------------
			cmd = new OleDbCommand();
			cmd.Connection = mcn;
			cmd.CommandText = "DELETE FROM Delivery WHERE DeliveryId = ?";
			cmd.Parameters.Add(new OleDbParameter("Original_DeliveryId", OleDbType.Integer, 4, System.Data.ParameterDirection.Input, false, ((System.Byte)(0)), ((System.Byte)(0)), "DeliveryId", System.Data.DataRowVersion.Original, null));
			mdaDelivery.DeleteCommand = cmd;
		}

		private void BuildDeliveryItemDataAdapter()
		{
			OleDbCommand cmd = null;

			mdaDeliveryItem = new OleDbDataAdapter();
			mdaDeliveryItem.TableMappings.AddRange(new DataTableMapping[] {
				new DataTableMapping("Table", "DeliveryItem", new DataColumnMapping[] {
				new DataColumnMapping("OrderItemId", "OrderItemId"),
				new DataColumnMapping("DeliveryId", "DeliveryId"),
				new DataColumnMapping("Qty", "Qty")})});

			// ------------------ SELECT -------------------
			cmd = new OleDbCommand();
			cmd.Connection = mcn;
			cmd.CommandText = "SELECT * FROM DeliveryItem";
			mdaDeliveryItem.SelectCommand = cmd;

			// ------------------ INSERT -------------------
			cmd = new OleDbCommand();
			cmd.Connection = mcn;
			cmd.CommandText = "INSERT INTO DeliveryItem VALUES (?, ?, ?)";
			cmd.Parameters.Add(new OleDbParameter("OrderItemId", OleDbType.Integer, 4, "OrderItemId"));
			cmd.Parameters.Add(new OleDbParameter("DeliveryId", OleDbType.Integer, 4, "DeliveryId"));
			cmd.Parameters.Add(new OleDbParameter("Qty", OleDbType.Integer, 4, "Qty"));
			mdaDeliveryItem.InsertCommand = cmd;

			// ------------------ UPDATE -------------------
			cmd = new OleDbCommand();
			cmd.Connection = mcn;
			cmd.CommandText = "UPDATE DeliveryItem SET Qty = ? WHERE (OrderItemId = ?) AND (DeliveryId = ?)";
			cmd.Parameters.Add(new OleDbParameter("Qty", OleDbType.Integer, 4, "Qty"));
			cmd.Parameters.Add(new OleDbParameter("Original_OrderItemId", OleDbType.Integer, 4, System.Data.ParameterDirection.Input, false, ((System.Byte)(0)), ((System.Byte)(0)), "OrderItemId", System.Data.DataRowVersion.Original, null));
			cmd.Parameters.Add(new OleDbParameter("Original_DeliveryId", OleDbType.Integer, 4, System.Data.ParameterDirection.Input, false, ((System.Byte)(0)), ((System.Byte)(0)), "DeliveryId", System.Data.DataRowVersion.Original, null));
			mdaDeliveryItem.UpdateCommand = cmd;

			// ------------------ DELETE -------------------
			cmd = new OleDbCommand();
			cmd.Connection = mcn;
			cmd.CommandText = "DELETE FROM DeliveryItem WHERE (OrderItemId = ?) AND (DeliveryId = ?)";
			cmd.Parameters.Add(new OleDbParameter("Original_OrderItemId", OleDbType.Integer, 4, System.Data.ParameterDirection.Input, false, ((System.Byte)(0)), ((System.Byte)(0)), "OrderItemId", System.Data.DataRowVersion.Original, null));
			cmd.Parameters.Add(new OleDbParameter("Original_DeliveryId", OleDbType.Integer, 4, System.Data.ParameterDirection.Input, false, ((System.Byte)(0)), ((System.Byte)(0)), "DeliveryId", System.Data.DataRowVersion.Original, null));
			mdaDeliveryItem.DeleteCommand = cmd;
		}


		public CommunicatorSqlServer(OleDbConnection cn)
		{
			mcn = cn;

			BuildCustomerDataAdapter();
			BuildOrderDataAdapter();
			BuildProductDataAdapter();
			BuildOrderItemDataAdapter();
			BuildDeliveryDataAdapter();
			BuildDeliveryItemDataAdapter();
		}


		override public DS GetDataSet()
		{
			DS ds = new DS();

			// All data in the database is supposed to have already passed integrity check. Doing all
			// integrity check again slows things down.
			ds.EnforceConstraints = false;
			mcn.Open();

			mdaCustomer.Fill(ds.Customer);
			mdaOrder.Fill(ds.Order);
			mdaProduct.Fill(ds.Product);
			mdaOrderItem.Fill(ds.OrderItem);
			mdaDelivery.Fill(ds.Delivery);
			mdaDeliveryItem.Fill(ds.DeliveryItem);

			mcn.Close();
			ds.EnforceConstraints = true;
			return ds;
		}


		override public void Update(DS ds)
		{
			mcn.Open();

			mdaCustomer.Update(ds.Customer.Select("", "", DataViewRowState.Added));
			mdaOrder.Update(ds.Order.Select("", "", DataViewRowState.Added));
			mdaProduct.Update(ds.Product.Select("", "", DataViewRowState.Added));
			mdaOrderItem.Update(ds.OrderItem.Select("", "", DataViewRowState.Added)); 
			mdaDelivery.Update(ds.Delivery.Select("", "", DataViewRowState.Added));
			mdaDeliveryItem.Update(ds.DeliveryItem.Select("", "", DataViewRowState.Added));

			mdaCustomer.Update(ds.Customer.Select("", "", DataViewRowState.ModifiedCurrent));
			mdaOrder.Update(ds.Order.Select("", "", DataViewRowState.ModifiedCurrent));
			mdaProduct.Update(ds.Product.Select("", "", DataViewRowState.ModifiedCurrent));
			mdaOrderItem.Update(ds.OrderItem.Select("", "", DataViewRowState.ModifiedCurrent));
			mdaDelivery.Update(ds.Delivery.Select("", "", DataViewRowState.ModifiedCurrent));
			mdaDeliveryItem.Update(ds.DeliveryItem.Select("", "", DataViewRowState.ModifiedCurrent));

			mdaDeliveryItem.Update(ds.DeliveryItem.Select("", "", DataViewRowState.Deleted));
			mdaDelivery.Update(ds.Delivery.Select("", "", DataViewRowState.Deleted));
			mdaOrderItem.Update(ds.OrderItem.Select("", "", DataViewRowState.Deleted));
			mdaProduct.Update(ds.Product.Select("", "", DataViewRowState.Deleted));
			mdaOrder.Update(ds.Order.Select("", "", DataViewRowState.Deleted));
			mdaCustomer.Update(ds.Customer.Select("", "", DataViewRowState.Deleted));

			mcn.Close();
		}

	}
}
